/* I ett tidigare exempel i avsnitt 2 fanns ett program med ett pussel.
   Då användes endast klick, för att markera och flytta bilder. Detta
   program är nu modifierat, så att det istället används drag-and-drop. */
// --------------------------------------------------
// Globala variabler och konstanter
const nrOfPuzzles = 6;  // Antal pussel i img-mappen
let puzzleNr;           // Aktuellt pussel
let pieceElems;         // Array med element för pusselbitar
let boxElems;           // Array med element för rutor i pusslet
let msgElem;            // Element för meddelande
// --------------------------------------------------
// Initiering av globala variabler och händelser.
function init() {
    pieceElems = document.querySelectorAll("#pieces img");
    boxElems = document.querySelectorAll("#puzzle img");
    for (let i = 0; i < 4; i++) {
        pieceElems[i].addEventListener("dragstart", dragStart);
        pieceElems[i].draggable = false;
        boxElems[i].addEventListener("dragstart", dragStart);
        boxElems[i].draggable = false;
    }
    msgElem = document.querySelector("#message");
    document.querySelector("#newPuzzleBtn").addEventListener("click", newPuzzle);
} // Slut init
window.addEventListener("load", init);
// --------------------------------------------------
// Välj slumpmässigt ett pussel och lägg ut pusselbitarna i den övre raden.
// Ta också bort bilderna i föregående pussel.
function newPuzzle() {
    puzzleNr = Math.floor(6 * Math.random()) + 1;
    let imgNrs = [1, 2, 3, 4];
    for (let i = 0; i < pieceElems.length; i++) {
        let r = Math.floor(imgNrs.length * Math.random());
        pieceElems[i].src = "img/puzzles/" + puzzleNr + "/" + imgNrs[r] + ".jpg";
        imgNrs.splice(r, 1); // Ta bort valt nummer, så att det inte kan väljas igen
        pieceElems[i].draggable = true;
    }
    for (let i = 0; i < boxElems.length; i++) {
        boxElems[i].src = "img/puzzles/empty.png";
        boxElems[i].draggable = false;
    }
    msgElem.innerText = "";
} // Slut newPuzzle
// --------------------------------------------------
// Hantera en drag-operation
function dragStart(e) {
    let dragElem = this;
    let dropElems = document.querySelectorAll("#puzzle img");
    dragElem.addEventListener("dragend", dragEnd);
    for (let i = 0; i < dropElems.length; i++) {
        dropElems[i].addEventListener("dragover", dropZone);
        dropElems[i].addEventListener("dragenter", dropZone);
        dropElems[i].addEventListener("dragleave", dropZone);
        dropElems[i].addEventListener("drop", dropZone);
    }
    // ----------
    // Avsluta drag-operationen
    function dragEnd(e) {
        dragElem.removeEventListener("dragend", dragEnd);
        for (let i = 0; i < dropElems.length; i++) {
            dropElems[i].removeEventListener("dragover", dropZone);
            dropElems[i].removeEventListener("dragenter", dropZone);
            dropElems[i].removeEventListener("dragleave", dropZone);
            dropElems[i].removeEventListener("drop", dropZone);
        }
    } // Slut dragEnd
    // ----------
    // Hantera händelser för drop zones
    function dropZone(e) {
        e.preventDefault();
        let dropElem = this;
        switch (e.type) {
            case "dragenter":
                dropElem.classList.add("hiliteDropZone");
                break;
            case "dragleave":
                dropElem.classList.remove("hiliteDropZone");
                break;
            case "drop":
                dropElem.classList.remove("hiliteDropZone");
                if (!dropElem.src.endsWith("empty.png")) { // Rutan är ej tom
                    let tempUrl = dropElem.src;
                    dropElem.src = dragElem.src;
                    dragElem.src = tempUrl;
                }
                else {
                    dropElem.src = dragElem.src;
                    dropElem.draggable = true;
                    dragElem.src = "img/puzzles/empty.png";
                    dragElem.draggable = false;
                }
                checkPuzzle(); // Kontrollera om pusslet är klart
        } // Slut switch
    } // Slut dropZone
    // ----------
} // Slut dragStart
// --------------------------------------------------
// Kontrollerar alla rutor i pusslet. Ligger bilderna i nummerordning, är pusslet klart.
function checkPuzzle() {
    for (let i = 0; i < boxElems.length; i++) {
        let url = boxElems[i].src;
        let nr = url.substring(url.lastIndexOf("/") + 1, url.lastIndexOf("."));
        if (nr != i + 1) { // Bilden på fel plats
            return;
        }
    }
    msgElem.innerText = "Grattis, pusslet är klart.";
} // Slut checkPuzzle
// --------------------------------------------------
