/* Exemplet visar användning av drag-and-drop där ord ska dras från en
   lista till ett antal bilder. I detta fall handlar det om stadsnamn
   och bilder på svenska städer. Variabelnamnen är dock valda mer generellt
   och kallas word och img, så innehållet kan lätt bytas till andra ord och
   bilder. Programmet kan användas som ett kunskapstest eller ett spel. */
// --------------------------------------------------
// Globala variabler och konstanter
const allNrs = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19]; // Index till allWords samt namn på bildfil
const allWords = ["Borgholm", "Gränna", "Gävle", "Göteborg", "Halmstad", "Jönköping", "Kalmar", "Karlskrona", "Kiruna", "Ljungby", "Malmö", "Norrköping", "Skara", "Stockholm", "Sundsvall", "Umeå", "Visby", "Västervik", "Växjö", "Örebro"]; // Namn på alla städer
let startBtn;		// Knapp för att starta spelet/testet
let checkBtn;       // Knapp för att kontrollera svaren
let wordElems;		// Array med li-element för orden
let imgElems;		// Array med img-element för bilderna
let answerElems;	// Array med element för för de ord som dras till bilderna
let msgElem; 		// Element för utskrift av meddelanden
// --------------------------------------------------
// Initiering då webbsidan är inläst
function init() {
    // Referenser till element i gränssnittet
    startBtn = document.querySelector("#startBtn");
    checkBtn = document.querySelector("#checkBtn");
    wordElems = document.querySelectorAll("#wordList li");
    imgElems = document.querySelectorAll("#imgList img");
    answerElems = document.querySelectorAll(".answer");
    msgElem = document.querySelector("#message");
    // Lägg på händelsehanterare
    startBtn.addEventListener("click", startGame);
    checkBtn.addEventListener("click", checkAnswers);
    for (let i = 0; i < wordElems.length; i++) { // Händelsehanterare för drag av ord i listan
        wordElems[i].addEventListener("dragstart", dragStart);
        wordElems[i].draggable = false;
    }
    for (let i = 0; i < answerElems.length; i++) { // Händelsehanterare för drag av ord vid bilderna
        answerElems[i].addEventListener("dragstart", dragStart);
        answerElems[i].draggable = false;
    }
    // Aktivera/inaktivera knappar
    startBtn.disabled = false;
    checkBtn.disabled = true;
} // Slut init
window.addEventListener("load", init);
// --------------------------------------------------
// Initiera "spelet". Välj ord slumpmässigt. Visa ord och bilder.
function startGame() {
    let copyNrs = allNrs.slice(0);
    let words = [];	// Array med de åtta ord som ska finnas i listan (ska sorteras i bokstavsordning)
    for (let i = 0; i < 4; i++) { // Fyra första ord, som även ska visas som bilder
        let r = Math.floor(copyNrs.length * Math.random());
        let ix = copyNrs[r];
        imgElems[i].src = "img/towns/" + ix + ".jpg";
        imgElems[i].setAttribute("data-ix", ix); // Behövs då svaren ska kontrolleras
        copyNrs.splice(r, 1);
        words.push(allWords[ix]);
    }
    for (let i = 0; i < 4; i++) { // Ytterligare fyra ord, fast inga bilder denna gång
        let r = Math.floor(copyNrs.length * Math.random());
        let ix = copyNrs[r];
        copyNrs.splice(r, 1);
        words.push(allWords[ix]);
    }
    words.sort(); // Sortera orden i bokstavsordning
    for (let i = 0; i < wordElems.length; i++) { // Visa orden
        wordElems[i].innerText = words[i];
        wordElems[i].draggable = true;
    }
    for (let i = 0; i < answerElems.length; i++) { // Rensa från förra omgången av spelet
        answerElems[i].innerHTML = "";
        answerElems[i].draggable = true;
    }
    msgElem.innerText = ""; // Töm elementet för meddelanden
    // Aktivera/inaktivera knappar
    startBtn.disabled = true;
    checkBtn.disabled = false;
} // Slut startGame
// --------------------------------------------------
// Hantera en drag-operation
function dragStart(e) {
    let dragElem = this;
    dragElem.addEventListener("dragend", dragEnd);
    for (let i = 0; i < imgElems.length; i++) {
        imgElems[i].addEventListener("dragover", dropZone);
        imgElems[i].addEventListener("drop", dropZone);
    }
    // ----------
    // Avsluta drag-operationen
    function dragEnd(e) {
        dragElem.removeEventListener("dragend", dragEnd);
        for (let i = 0; i < imgElems.length; i++) {
            imgElems[i].removeEventListener("dragover", dropZone);
            imgElems[i].removeEventListener("drop", dropZone);
        }
    } // Slut dragEnd
    // ----------
    // Hantera händelser för drop zones
    function dropZone(e) {
        e.preventDefault();
        let dropElem = this;
        if (e.type == "drop") {
            let answerElem = dropElem.nextElementSibling; // p-elementet intill bilden
            if (answerElem.innerText != "") { // Det finns redan ett ord
                moveBackToList(answerElem.innerText);
            }
            answerElem.innerText = dragElem.innerText;
            dragElem.innerText = "";
        }
    } // Slut dropZone
    // ----------
    function moveBackToList(word) {
        for (let i = 0; i < wordElems.length; i++) {  // Leta efter ...
            if (wordElems[i].innerText == "") {       // ... tom plats ...
                wordElems[i].innerText = word;   // ... och lägg tillbaks ordet
                break;
            }
        }
    } // Slut moveBackToList
} // Slut dragStart
// --------------------------------------------------
// Kontrollera användarens svar och visa de korrekta svaren
function checkAnswers() {
    // Kolla först att det dragits ett ord till alla bilder
    for (let i = 0; i < answerElems.length; i++) {
        if (answerElems[i].innerText == "") {
            msgElem.innerText = "Dra först ord till alla bilder.";
            return;
        }
    }
    // Ta bort möjligheten att dra orden
    for (let i = 0; i < wordElems.length; i++) {// Ord i listan
        wordElems[i].draggable = false;
    }
    for (let i = 0; i < answerElems.length; i++) { // Ord vid bilderna
        answerElems[i].draggable = false;
    }
    // Kontrollera antal rätt (poäng) och fyll i de rätta svaren
    let points = 0;
    for (let i = 0; i < answerElems.length; i++) {
        let ix = imgElems[i].getAttribute("data-ix");
        if (answerElems[i].innerText == allWords[ix]) {
            points++;
        }
        answerElems[i].innerHTML += "<br>Rätt svar: " + allWords[ix];
    }
    msgElem.innerText = "Du hade " + points + " rätt.";
    startBtn.disabled = false;
    checkBtn.disabled = true;
} // Slut checkAnswers
// --------------------------------------------------
